package jigcell.project;

import java.beans.ExceptionListener;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * A ProjectFileHandler can retrieve and store the data associated with a ProjectFile.
 *
 * This code is licensed under the DARPA BioCOMP Open Source License.  See LICENSE for more details.
 *
 * @author Nicholas Allen
 */

public interface ProjectFileHandler {

   /**
    * Retrieves data from an input stream.
    *
    * @param input Input stream
    * @param listener Listener to notify of any recoverable exceptions (may be null)
    * @param mimeType Data encoding the input stream is expected to contain
    *
    * @throws ProjectException if no data can be loaded
    */

   Object load (InputStream input, ExceptionListener listener, String mimeType) throws ProjectException;

   /**
    * Stores data to an output stream.
    *
    * @param output Output stream
    * @param listener Listener to notify of any recoverable exceptions (may be null)
    * @param mimeType Data encoding the saver should use
    * @param o Object to store
    *
    * @throws ProjectException if no data can be saved
    */

   void save (OutputStream output, ExceptionListener listener, String mimeType, Object o) throws ProjectException;
}
