package jigcell.project;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
import javax.swing.JTable;
import javax.swing.JFileChooser;
import java.io.*;
import jigcell.modelbuilder.ModelBuilder;
import jigcell.modelbuilder.xModelBuilderFrame;
import java.beans.ExceptionListener;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

// add check at close for modified
// must read the version at the store somehow at loading an already existing project

// the file panel is created outside of initComponents because we need to add to it in load

public class ProjectManager extends javax.swing.JFrame {

	// gui related stuff
	private javax.swing.JMenuBar jMenuBar1;
	private javax.swing.JPanel pane;

	// project related stuff
	private DiskStore storage = null;
	private Project workingProject = null;
	private boolean modified = false;
	private GroupTable projectView = null;
	private ProjectFile selectedFile = null;

	public ProjectManager() {

		ProjectChooser selectProject = new ProjectChooser(new File(System.getProperty("user.dir")));
		selectProject.setModal(true);
		selectProject.show();
		if (selectProject.exitStatus == selectProject.CANCEL) System.exit(0);

		// see if we have to create a new project complete with the folder
		if (selectProject.exitStatus == selectProject.NEW)
			newProject(selectProject.getSelectedProject());
		else {
			// we must load the project
			loadProject(selectProject.getSelectedProject());
		}
		initComponents();
		this.setVisible(true);
	}

	private void closeProject() {
		this.setVisible(false);
		ProjectChooser selectProject = new ProjectChooser(new File(System.getProperty("user.dir")));
		selectProject.setModal(true);
		selectProject.show();
		if (selectProject.exitStatus != selectProject.OK) System.exit(0);
		initComponents();
		this.setVisible(true);
	}

	private void initComponents() {
		
		// construct the menus
		jMenuBar1 = new javax.swing.JMenuBar();

		JMenu jMenuFile = new JMenu();
		jMenuFile.setText("File");
		JMenu jMenuFileNew = new JMenu();
		JMenuItem jMenuFileNewModel = new JMenuItem("Model");
		JMenuItem jMenuFileNewRun = new JMenuItem("Run Set");
		JMenuItem jMenuFileNewExperiment = new JMenuItem("Experiment Set");
		JMenuItem jMenuFileNewObjective = new JMenuItem("Objective Function");
		JMenuItem jMenuFileNewEstimator = new JMenuItem("Parameter Estimator Data");
		JMenuItem jMenuFileImport = new JMenuItem();
		JMenuItem jMenuFileExport = new JMenuItem();
		JMenuItem jMenuFileSave = new JMenuItem();
		JMenuItem jMenuFileSaveAs = new JMenuItem();
		JMenuItem jMenuFileClose = new JMenuItem("Close Project");
		JMenuItem jMenuFileExit = new JMenuItem();
		jMenuFileNew.setText("New");
		jMenuFileImport.setText("Import File to Project");
		jMenuFileExport.setText("Export File");
		jMenuFileSave.setText("Save Project");
		jMenuFileSaveAs.setText("Save Project As");
		jMenuFileExit.setText("Exit");
		jMenuFileNewModel.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent e) {
				newModel();
			}
		});
		jMenuFileImport.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent e) {
				importFile();
			}
		});
		jMenuFileExport.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent e) {
				exportFile();
			}
		});
		jMenuFileClose.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent e) {
				closeProject();
			}
		});
		jMenuFileExit.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent e) {
				exitForm(null);
			}
		});
		jMenuFile.setMnemonic(KeyEvent.VK_F);

		// add the menus
		jMenuFileNew.add(jMenuFileNewModel);
		jMenuFileNew.add(jMenuFileNewRun);
		jMenuFileNew.add(jMenuFileNewExperiment);
		jMenuFileNew.add(jMenuFileNewObjective);
		jMenuFileNew.add(jMenuFileNewEstimator);
		jMenuFile.add(jMenuFileNew);
		jMenuFile.add(jMenuFileSave);
		jMenuFile.add(jMenuFileSaveAs);
		jMenuFile.addSeparator();
		jMenuFile.add(jMenuFileImport);
		jMenuFile.add(jMenuFileExport);
		jMenuFile.addSeparator();
		jMenuFile.add(jMenuFileClose);
		jMenuFile.addSeparator();
		jMenuFile.add(jMenuFileExit);
		jMenuBar1.add(jMenuFile);
		setJMenuBar(jMenuBar1);

		// set up the look and feel
		JFrame.setDefaultLookAndFeelDecorated(true);

		// set up the initial frame
		try {
			this.setTitle("Jigcell Project Manager:" + workingProject.getName());
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "Error in Project", "Project error", JOptionPane.ERROR_MESSAGE);
		}
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		pane = new javax.swing.JPanel();
		pane.setPreferredSize(new Dimension(800, 300));

		// application launch
		JPanel buttonsView = new JPanel(new GridLayout(0,1));
		JButton mb = new JButton("Model Builder");
		JButton rm = new JButton("Run Manager");
		JButton cmp = new JButton("Comparator");
		JButton pe = new JButton("Parameter Estimator");
		mb.addActionListener(new ActionListener()  {
			public void actionPerformed(ActionEvent e) {
				startModelbuilder();
			}
		});
		buttonsView.add(mb);buttonsView.add(rm);buttonsView.add(cmp);buttonsView.add(pe);

		final JTable table = new JTable(projectView);
		// give the table a selection so we can click on rows and tools and have it start up
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		ListSelectionModel rowSM = table.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if (e.getValueIsAdjusting()) return;
				ListSelectionModel lsm = (ListSelectionModel)e.getSource();
				if (!lsm.isSelectionEmpty()) {
					int selectedRow = lsm.getMinSelectionIndex();
					// get the file at that location and see what it is
					selectedFile = projectView.getFileAt(selectedRow);
				} else selectedFile = null;
			}
		});


		JScrollPane tableView = new JScrollPane(table);
		tableView.setPreferredSize(new Dimension(400, 300));

		// create the split panes
		// JSplitPane projectView = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, tableView, fileView);
		// JSplitPane buttons = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, buttonsView, projectView);
		JSplitPane buttons = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, buttonsView, tableView);
		// .setOneTouchExpandable(true);
	        // splitPane.setDividerLocation(150);
		this.getContentPane().add(buttons, BorderLayout.CENTER);
		this.pack();
	}

	private void newProject(String projectName) {
		workingProject = null;
		try {
			// Create the project
			File dir = new File(System.getProperty("user.dir"));
                        String projectPath = dir.getCanonicalPath() + dir.separator + "projects" + dir.separator + projectName;
                        File projectDir = new File(projectPath);
			projectDir.mkdir();
			System.out.println(projectDir.getCanonicalPath());
			workingProject = new Project(projectName, DiskStore.class, DiskStore.createDiskStoreData(projectDir));
			// create the store
			storage = new DiskStore(projectName, "1.0", workingProject);

			// create the project table panel
			projectView = new GroupTable();
			projectView.setProjectManager(this);
			projectView.setProject(workingProject);
		} catch (Exception e) {
			e.printStackTrace(System.err); 
			JOptionPane.showMessageDialog(this, "Error: Cannot create Project", "Project error", JOptionPane.ERROR_MESSAGE);
		}
	}

	private void loadProject(String projectName) {
		workingProject = null;
		try {
			// open the project by creating it
			File dir = new File(System.getProperty("user.dir"));
			String projectPath = dir.getCanonicalPath() + dir.separator + "projects" + dir.separator + projectName;
			File projectDir = new File(projectPath);
			workingProject = new Project(projectName, DiskStore.class, DiskStore.createDiskStoreData(projectDir));
			storage = new DiskStore(projectName, "1.0", workingProject);

			// create the project table panel
			projectView = new GroupTable();
			projectView.setProjectManager(this);
			projectView.setProject(workingProject);
			// load the list of files in the interface
			java.util.List projectFiles = workingProject.getProjectFiles();
			System.out.println("Reported " + projectFiles.size() + " files in the project (index)");
			for (int i = 0; i < projectFiles.size(); i++) {
				// projectView.addFile((ProjectFile)projectFiles.get(i)); // we don't need to add it because the table model takes care of that
				// see if we need a handler
				try {
					ProjectFile newFile = (ProjectFile)projectFiles.get(i);
					newFile.registerHandler(newFile.getMIMEType(newFile.getVersion()), new JigcellFileHandler());
				} catch (Exception e) {System.out.println("Already have handler");}
			}
				
		} catch (Exception e) {
			e.printStackTrace(System.err); 
			JOptionPane.showMessageDialog(this, "Error: Cannot open Project", "Project error", JOptionPane.ERROR_MESSAGE);
		}
	}

	/*
	private void createNodes(DefaultMutableTreeNode root) {
		// get the list of files in the project
		java.util.List files = null;
		try {
			files = workingProject.getProjectFiles();
		} catch (Exception e) {
			e.printStackTrace(System.err);
			JOptionPane.showMessageDialog(this, "Error: Cannot read files in project", "Project error", JOptionPane.ERROR_MESSAGE);
		}
		// because the project stuff has really turned out to be very stupid, we count all files in this demo as model files, leaving
		// the stuff about mime types at a later date
		DefaultMutableTreeNode target = null;
		DefaultMutableTreeNode entry = null;

		target = new DefaultMutableTreeNode("Models");
		root.add(target);
		ListIterator nodes = files.listIterator();
		if (!nodes.hasNext()) {
			entry = new DefaultMutableTreeNode("New Model");
			target.add(entry);
		} else
			while (nodes.hasNext()) {
				ProjectFile file = (ProjectFile)nodes.next();
				try {
					entry = new DefaultMutableTreeNode(file.getName());
					target.add(entry);
				} catch (Exception e) {e.printStackTrace(System.err);}
			} 
		
		target = new DefaultMutableTreeNode("Runs");
		root.add(target);
		entry = new DefaultMutableTreeNode("New Run File");
		target.add(entry);
		
		target = new DefaultMutableTreeNode("Experiments");
		root.add(target);
		entry = new DefaultMutableTreeNode("New Experiment");
		target.add(entry);
		
		target = new DefaultMutableTreeNode("Objective Functions");
		root.add(target);
		entry = new DefaultMutableTreeNode("New Objective Function");
		target.add(entry);
		
		target = new DefaultMutableTreeNode("Parameter Estimator Data");
		root.add(target);
		entry = new DefaultMutableTreeNode("New Entry");
		target.add(entry);	
	}*/

	private void exitForm(java.awt.event.WindowEvent evt) {
		System.exit(0);
	}

	private void newModel() {
		ModelBuilder work = new ModelBuilder(new xModelBuilderFrame());
		work.setProject(workingProject);
		work.frame.show();
	}

	public void openModel(ProjectFile toOpen) {
		System.out.println("Opening project file");
		ModelBuilder work = new ModelBuilder(new xModelBuilderFrame());
		work.setProject(workingProject);
		work.openProjectFile(toOpen);
		work.frame.show();
	}

	private void startModelbuilder() {
		try {
			if (selectedFile == null || selectedFile.getMIMEType(selectedFile.getVersion()).compareTo("SBML") != 0)
				newModel();
			else openModel(selectedFile);
		} catch (Exception e) {e.printStackTrace(System.err); }
	}

	private void exportFile() {
		try {
			if (selectedFile != null) {
				String currentType = selectedFile.getMIMEType(selectedFile.getVersion());
				// give the user a JFileChooser
				JFileChooser fc = new JFileChooser();
				// create the extension out of the mime type
				ExampleFileFilter ex2 = new ExampleFileFilter();
				ex2.addExtension(currentType.toLowerCase());
				ex2.setDescription(currentType + " Files");
				fc.addChoosableFileFilter(ex2);
				
				// for now set the folder to the user working dir
				fc.setCurrentDirectory(new File(System.getProperty("user.dir")));
				int returnvalue = fc.showSaveDialog(this);
				if(JFileChooser.APPROVE_OPTION == returnvalue){// file chosen
					String fileName = fc.getSelectedFile().getPath();
					String actual = fileName.endsWith("." + currentType.toLowerCase())?fileName:fileName + "." + currentType.toLowerCase();

					// to save to the external file we need to LOAD from the project file
					java.io.RandomAccessFile file = new java.io.RandomAccessFile(actual,"rw");
					file.writeBytes((String)selectedFile.load(null, selectedFile.getVersion()));
					file.close();
				} // else there isn;t much we can do

			}
		} catch (Exception e) {e.printStackTrace(System.err); }
	}

	private void importFile() {
		// give the user a JFileChooser
		JFileChooser f = new JFileChooser();

		// set up the filters for the type of files that we can import, which will
		// eventually be all the files recognized by tools in jigcell


		// start with the model builder
		ExampleFileFilter ex = new ExampleFileFilter();
		ex.addExtension("mec");					// old style models
		ex.setDescription("Mechanism Files");

		ExampleFileFilter ex2 = new ExampleFileFilter();	// new style models
		ex2.addExtension("sbml");
		ex2.setDescription("SBML Level 2 Files");

		// add them up
		f.addChoosableFileFilter(ex);
		f.addChoosableFileFilter(ex2);
		f.setFileFilter(ex2);					// i think what we are looking for by default

		// look in the default user directory
		f.setCurrentDirectory(new File(System.getProperty("user.dir")));
		int returnvalue = f.showOpenDialog(this);

		if(JFileChooser.APPROVE_OPTION == returnvalue){// file chosen
			String filename = f.getSelectedFile().getPath();
			if(filename.endsWith(".mec") || filename.endsWith(".sbml")){
				// add the model in under the "models" node, but first make a file for it
				try {
					String chopped = new String(f.getSelectedFile().getName());
					chopped = chopped.replaceAll("\\.sbml", "");
					chopped = chopped.replaceAll("\\.mec", "");
					ProjectFile newmodel = new ProjectFile(chopped, workingProject);
					// see if the project has a handler for this
					try {
						newmodel.registerHandler("SBML", new JigcellFileHandler());
					} catch (Exception e) {System.out.println("Already have handler");}
					java.io.RandomAccessFile file = new java.io.RandomAccessFile(f.getSelectedFile(),"r");
					byte b[] = new byte[(int)file.length()];
					file.readFully(b);
					file.close();
					String data = new String(b);
					newmodel.save(null, "First imported version", "SBML", data);
					projectView.addFile(newmodel);
				} catch (Exception e) {
					e.printStackTrace(System.err);
					JOptionPane.showMessageDialog(this, "Error: Cannot read files in project", "Project error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	public class JigcellFileHandler implements ProjectFileHandler {

		// we expect the models to be strings 
		public void save (OutputStream output, ExceptionListener listener, String mimeType, Object o) throws ProjectException{
			if (mimeType == "SBML") {
				DataOutputStream out = new DataOutputStream(output);
				try {out.writeBytes((String)o);}
				catch (Exception e) {throw new ProjectException();}
			}
		}

		public Object load (InputStream input, ExceptionListener listener, String mimeType) throws ProjectException {
			String content = new String();
			DataInputStream in = new DataInputStream(input);
			try { byte b[] = new byte[input.available()]; in.read(b, 0, input.available()); content = new String(b); }
			catch (Exception e) { e.printStackTrace(System.err); throw new ProjectException();}
			return content;
		}

	}

	public static void main(String s[]) {
		new ProjectManager();
	}
}
